from PIL import Image
import numpy as np
from time import time

# načtení obrázku pomocí knihovny Pillow
img = Image.open('kvetina.jpg')

# převedení obrázku na Numpy-pole
data = np.asarray(img)
# tvar (rozměry) vyrobeného Numpy-pole
h, w, d = data.shape   # (375, 500, 3)

# a) výpočet zanořenými smyčkami operací na jednotlivých pixelech
new_data = np.zeros([h, w], dtype=data.dtype)
t = time()
for i in range(h):
    for j in range(w):
        r, g, b = data[i, j]
        new_data[i, j] = 0.299 * r + 0.587 * g + 0.114 * b
print(time() - t)   # 1.1509654521942139
# https://pillow.readthedocs.io/en/stable/handbook/concepts.html#concept-modes
#Image.fromarray(new_data, 'L').show()

# b) výpočet pomocí broadcastingu vektorově na celém poli najednou
t = time()
R, G, B = data[:, :, 0], data[..., 1], data[:, :, 2]    # :,: == ...
new_data_2 = R * 0.299 + G * 0.587 + B * 0.114  # float
new_data_2 = new_data_2.astype(data.dtype)      # uint8
print(time() - t)   # 0.008
print(new_data_2[0, 0]) # 65 (s voláním bez ".astype()" -> 65.14599999999999)
Image.fromarray(new_data_2, 'L').show()

# porovnání
print(np.array_equal(new_data, new_data_2))     # True
